package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 3;

use Data::Dumper;
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::Product;

## no critic qw(ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->test_get;
    $self->test_get_master;
    $self->test_get_sub;
}

sub test_get {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::Product->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyPetrol');
    my $SiteLocale = LoadRootObject->siteLocale($LocaleID);
    my $TaxType = 'GROSS';
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $SfBaseUrl = $self->shop->get('WebUrl') ;

    my $hData = $Resource->get( $Product, $LocaleID, $CurrencyID,  );

    is_deeply( $hData, {
        availability => 'OnStock',
        availabilityText => '',
        deliveryWeight => undef,
        description => undef,
        forSale => 'true',
        images => [
        ],
        specialOffer => 'false',
        name => 'MyPetrol',
        priceInfo => {
            basePrice => undef,
            depositPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            ecoParticipationPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            manufacturerPrice => _Price( $SiteLocale, 0.6, $CurrencyID, $TaxType ),
            price => _Price( $SiteLocale, 1.19, $CurrencyID, $TaxType ),
            priceWithDeposits => _Price( $SiteLocale, 1.19, $CurrencyID, $TaxType ),
            quantity => {
                amount => 1,
                unit => 'l'
            },
        },
        productId => $Product->guid,
        shortDescription => undef,
        shippingMethodsRestrictedTo => undef,
        sfUrl => $SfBaseUrl.'/?ObjectPath=/Test/TestShop/Products/MyPetrol',
        links => [
            {
                rel => 'self',
                href => $SSLDomain.'/api/products/'.$Product->guid,
            },
            {
                rel => 'slideshow',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/slideshow',
            }
        ],
        energyLabelsString => undef,
        energyLabelSourceFile => undef,
        productDataSheet => undef,
        productNumber => 'MyPetrol',
        manufacturer => undef,
        upcEan => undef,
    }, 'GET MyPetrol' ) or diag Dumper($hData);
}

sub test_get_master {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::Product->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket');
    my $SiteLocale = LoadRootObject->siteLocale($LocaleID);
    my $TaxType = 'GROSS';
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $SfBaseUrl = $self->shop->get('WebUrl') ;

    my $hData = $Resource->get( $Product, $LocaleID, $CurrencyID );

    is_deeply( $hData, {
        availability => undef,
        availabilityText => '',
        forSale => 'false',
        deliveryWeight => undef,
        description => undef,
        shortDescription => undef,
        images => [{
           'url' => $SSLDomain.$Product->get('WebPath').'/myjacket.png',
           'classifier' => 'Small'
        }],
        specialOffer => 'false',
        name => 'MyJacket',
        priceInfo => {
            basePrice => undef,
            depositPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            ecoParticipationPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            manufacturerPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            price => _Price( $SiteLocale, 199.95, $CurrencyID, $TaxType ),
            priceWithDeposits => _Price( $SiteLocale, 199.95, $CurrencyID, $TaxType ),
            quantity => {
                amount => 1,
                unit => 'piece(s)'
            },
        },
        productId => $Product->guid,
        shortDescription => undef,
        shippingMethodsRestrictedTo => undef,
        sfUrl => $SfBaseUrl.'/?ObjectPath=/Test/TestShop/Products/MyJacket',
        links => [
            {
                rel => 'self',
                href => $SSLDomain.'/api/products/'.$Product->guid,
            },
            {
                rel => 'custom-attributes',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/custom-attributes',
            },
            {
                rel => 'variations',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/variations',
            },
            {
                rel => 'lowest-price',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/lowest-price',
            },
        ],
        energyLabelsString => undef,
        energyLabelSourceFile => undef,
        productDataSheet => undef,
        productNumber => 'MyJacket',
        manufacturer => undef,
        upcEan => undef,
    }, 'GET MyJacket' ) or diag Dumper($hData);
}

sub test_get_sub {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::Product->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2');
    my $SiteLocale = LoadRootObject->siteLocale($LocaleID);
    my $TaxType = 'GROSS';
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $SfBaseUrl = $self->shop->get('WebUrl') ;

    my $hData = $Resource->get( $Product, $LocaleID, $CurrencyID,  );

    is_deeply( $hData, {
        availability => 'OnStock',
        availabilityText => '',
        forSale => 'true',
        deliveryWeight => undef,
        description => undef,
        images => [{
           'url' => $SSLDomain.$Product->get('WebPath').'/myjacket.png',
           'classifier' => 'Small'
        }],
        specialOffer => 'false',
        name => 'JacketBlack2',
        priceInfo => {
            basePrice => undef,
            depositPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            ecoParticipationPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            manufacturerPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            price => _Price( $SiteLocale, 189.95, $CurrencyID, $TaxType ),
            priceWithDeposits => _Price( $SiteLocale, 189.95, $CurrencyID, $TaxType ),
            quantity => {
                amount => 1,
                unit => 'piece(s)'
            },
        },
        productId => $Product->guid,
        shortDescription => undef,
        shippingMethodsRestrictedTo => undef,
        sfUrl => $SfBaseUrl.'/?ObjectPath=/Test/TestShop/Products/MyJacket/SubProducts/JacketBlack2',
        links => [
            {
                rel => 'self',
                href => $SSLDomain.'/api/products/'.$Product->guid,
            },
            {
                rel => 'custom-attributes',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/custom-attributes',
            },
            {
                rel => 'variations',
                href => $SSLDomain.'/api/products/'.$Product->superProduct->guid.'/variations',
            },
            {
                rel => 'master',
                href => $SSLDomain.'/api/products/'.$Product->superProduct->guid,
            },
        ],
        energyLabelsString => undef,
        energyLabelSourceFile => undef,
        productDataSheet => undef,
        productNumber => 'JacketBlack2',
        manufacturer => undef,
        upcEan => undef,
    }, 'GET JacketBlack2' ) or diag Dumper($hData);
}

sub _Price {
    my ($SiteLocale, $Amount, $CurrencyID, $TaxType) = @_;

    return undef unless defined $Amount;

    return {
        amount => $Amount,
        currency => $CurrencyID,
        formatted => defined $Amount ? $SiteLocale->formatMoney($Amount, $CurrencyID) : undef,
        taxType => $TaxType,
    };
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;
