#========================================================================================
# package      DE_EPAGES::Shippo::t::unit::AppStoreShippo
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for class AppStoreShippo.
#========================================================================================
package DE_EPAGES::Shippo::t::unit::AppStoreShippo;

use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Shippo::API::AppStoreShippo;
use DE_EPAGES::Test::API::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant APPID => '1234567890';

sub test {
    my $self = shift;
    $self->get_shippo_appid_positive();
    $self->check_shippo_app_installed_positive();
    $self->install_shippo_positive();
    $self->uninstall_shippo_positive();
    done_testing();
}

sub get_shippo_appid_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedAppStoreShippo = $self->_createAppStoreShippo();
    my $MockedShop           = $self->_createShop();
    is( $MockedAppStoreShippo->getShippoAppId($MockedShop), '1234567890', "$SubTestName - Shippo App Id is correct." );
    is( $self->{'mockedParams'}->{'AppType'},               'APP_SOAP',   "$SubTestName - AppType is APP_SOAP" );
    is( $self->{'mockedParams'}->{'Domain'}, 'goshippo.com', "$SubTestName - Domain is goshippo.com" );

    return;
}

sub check_shippo_app_installed_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedAppStoreShippo = $self->_createAppStoreShippo();
    my $MockedShop           = $self->_createShop();
    is( $MockedAppStoreShippo->isShippoAppInstalled( $MockedShop, ), '1', "$SubTestName - Shippo App is installed." );

    return;
}

sub install_shippo_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedAppStoreShippo = $self->_createAppStoreShippo();
    my $MockedShop           = $self->_createShop();
    my $MockedServlet        = $self->_createServlet( $MockedShop, APPID );

    is( $MockedAppStoreShippo->installShippoApp( $MockedServlet, $MockedShop ),
        '1', "$SubTestName - Shippo App has been successfully installed." );

    return;
}

sub uninstall_shippo_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedAppStoreShippo = $self->_createAppStoreShippo();
    my $MockedShop           = $self->_createShop();
    my $MockedServlet        = $self->_createServlet( $MockedShop, APPID );

    is( $MockedAppStoreShippo->uninstallShippoApp( $MockedServlet, $MockedShop ),
        '1', "$SubTestName - Shippo App has been successfully uninstalled." );

    return;
}

#----------------------------------------------------------------------------------------

sub _createAppStoreShippo {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Shippo::API::AppStoreShippo', $aParameterList );
    my $aParameterTypes = ['hashref'];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_createAppsClient', $self->_createAppsClient(),
        $aParameterTypes );

    $aParameterTypes = [ 'string', 'object', 'string' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_TestObject', '', $aParameterTypes );

    $aParameterTypes = ['string'];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_isValid', '1', $aParameterTypes );

    $aParameterTypes = [ 'string', 'string', 'string', 'string' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_LogShippingFulfillment', '', $aParameterTypes );

    $aParameterTypes = [ 'string', 'hashref' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_TriggerHook', '', $aParameterTypes );

    return $Mockify->getMockObject();
}

sub _createAppsClient {
    my $self = shift;

    my $MockedAppsClient = Test::MockObject->new();
    TestExistsMethod( 'DE_EPAGES::AppStore::API::Object::AppsClient', 'findAppIdByUrl' );
    $MockedAppsClient->mock(
        'findAppIdByUrl',
        sub {
            my $MockedSelf = shift;
            my ( $appType, $appUrl ) = @_;
            $self->{'mockedParams'}->{'AppType'} = $appType;
            $self->{'mockedParams'}->{'Domain'}  = $appUrl;

            return APPID;
        }
    );

    $MockedAppsClient->mock(
        'getAppLinks',
        sub {
            my $MockedSelf = shift;
            my ($appId) = @_;

            my $hLinks = { 'links' => [ { 'rel' => 'uninstallApp' } ] };

            return $hLinks;
        }
    );

    return $MockedAppsClient;
}

sub _createShop {
    my $self = shift;

    my $hAttributes = { 'ShopTypeRef' => $self->_createShopTypeRef() };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );

    return $Shop;
}

sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = { 'Alias' => 'Demo' };
    my $ShopTypeRef =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes );

    return $ShopTypeRef;
}

sub _createServlet {
    my $self = shift;
    my ( $Shop, $appId ) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    if ( isValid($appId) ) {
        $Mockify->addVar( 'appID', $appId, 'string' );
    }

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

1;
